/*
 * @Copyright 2010, MechSoft 
 * MechSoft, Mechanical and Software Solutions 
 * 
 * Licensed under the Apache License, Version 2.0 (the 
 * "License"); you may not use this file except in compliance 
 * with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.extjs.example;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.defne.jpa.EntityManagerUtil;
import org.defne.service.Message;
import org.defne.service.MessageFactory;
import org.defne.service.TransactionPolicy;
import org.defne.service.annotation.EntityManagerAttribute;
import org.defne.service.annotation.Operation;
import org.defne.service.annotation.Service;
import org.defne.service.annotation.TransactionAttribute;
import org.defne.utility.exception.DefneException;

@Service
@SuppressWarnings("unchecked")
public class SurveyService
{
    
    @Operation
    @TransactionAttribute(TransactionPolicy.WITH_TRANSACTION)
    @EntityManagerAttribute
    public static Message addSurvey(Message message) throws DefneException
    {
        Survey survey = message.getMessageParameter(Survey.class, ISurveyService.AddOperation.INPUT.SURVEY);
        EntityManager manager = EntityManagerUtil.getEntityManagerFromBag(message);
        
        manager.persist(survey);
        
        Message output = MessageFactory.newMessage();
        output.putMessageParameter(ISurveyService.AddOperation.OUTPUT.MESSAGE, "Survey is added with id : " + survey.getId());
        
        return output;
    }
    
    @Operation
    @TransactionAttribute(TransactionPolicy.NO_TRANSACTION)
    @EntityManagerAttribute
    public static Message listSurveys(Message message) throws DefneException
    {
        EntityManager manager = EntityManagerUtil.getEntityManagerFromBag(message);
        Query query = manager.createQuery("select s from Survey s");
        
        List<Survey> surveys = (List<Survey>) query.getResultList();
        
        Message oBag = MessageFactory.newMessage();
        oBag.putMessageParameter(ISurveyService.ListOperation.OUTPUT.LIST, surveys);
        
        return oBag;
    }
}
